@@ -1,6 +1,7 @@
 /*
-    Copyright 2008-2023
+    Copyright 2008-2024
         Matthias Ehmann,
+        Aaron Fenyes,
         Carsten Miller,
         Andreas Walter,
         Alfred Wassermann
@@ -32,11 +33,11 @@
  * Create linear spaces of dimension at least one,
  * i.e. lines and planes.
  */
-import JXG from '../jxg';
-import Const from '../base/constants';
-import Type from '../utils/type';
-import Mat from '../math/math';
-import Geometry from '../math/geometry';
+import JXG from '../jxg.js';
+import Const from '../base/constants.js';
+import Type from '../utils/type.js';
+import Mat from '../math/math.js';
+import Geometry from '../math/geometry.js';
 
 // -----------------------
 //  Lines
@@ -73,7 +74,7 @@ JXG.Line3D = function (view, point, direction, range, attributes) {
      * Direction which - together with a point - defines the line. Array of numbers or functions (of length 3) or function
      * returning array of length 3.
      *
-     * @type Array,Function
+     * @type Array|Function
      * @see JXG.Line3D#point
      */
     this.direction = direction;
@@ -132,13 +133,12 @@ JXG.extend(
                 }
             }
 
+            // Intersect the ray - if necessary - with the cube,
+            // i.e. clamp the line.
             r0 = Type.evaluate(r);
-            // TODO: test also in the finite case
-            if (Math.abs(r0) === Infinity) {
-                r = this.view.intersectionLineCube(p, d, r0);
-            }
+            r = this.view.intersectionLineCube(p, d, r0);
 
-            return [p[0] + d[0] * r0, p[1] + d[1] * r0, p[2] + d[2] * r0];
+            return [p[0] + d[0] * r, p[1] + d[1] * r, p[2] + d[2] * r];
         },
 
         update: function () {
@@ -148,6 +148,35 @@ JXG.extend(
         updateRenderer: function () {
             this.needsUpdate = false;
             return this;
+        },
+
+        projectCoords: function (p) {
+            var p0_coords = this.getPointCoords(0),
+                p1_coords = this.getPointCoords(1),
+                dir = [
+                    p1_coords[0] - p0_coords[0],
+                    p1_coords[1] - p0_coords[1],
+                    p1_coords[2] - p0_coords[2]
+                ],
+                diff = [
+                    p[0] - p0_coords[0],
+                    p[1] - p0_coords[1],
+                    p[2] - p0_coords[2]
+                ],
+                t = Mat.innerProduct(diff, dir) / Mat.innerProduct(dir, dir),
+                t_clamped = Math.min(Math.max(t, this.range[0]), this.range[1]),
+                c3d;
+
+            c3d = this.getPointCoords(t_clamped).slice();
+            c3d.unshift(1);
+            return c3d;
+        },
+
+        projectScreenCoords: function (pScr) {
+            var end0 = this.getPointCoords(0),
+                end1 = this.getPointCoords(1);
+
+            return this.view.projectScreenToSegment(pScr, end0, end1);
         }
     }
 );
@@ -219,7 +248,8 @@ JXG.createLine3D = function (board, parents, attributes) {
     var view = parents[0],
         attr, points,
         point, direction, range,
-        point1, point2, el;
+        point1, point2, endpoints,
+        el;
 
     attr = Type.copyAttributes(attributes, board.options, 'line3d');
 
@@ -236,10 +266,65 @@ JXG.createLine3D = function (board, parents, attributes) {
         direction = function () {
             return [point2.X() - point1.X(), point2.Y() - point1.Y(), point2.Z() - point1.Z()];
         };
-        range = [0, 1];
+        range = [0, 1]; // Segment by default
         el = new JXG.Line3D(view, point1, direction, range, attr);
+
+        // Create two shadow points that are the end points of the visible line.
+        // This is of relevance if the line has straightFirst or straightLast set to true, then
+        // endpoints differ from point1, point2.
+        // In such a case, the endpoints are the intersection of the line with the cube.
+        endpoints = Type.providePoints3D(
+            view,
+            [
+                [0, 0, 0],
+                [0, 0, 0]
+            ],
+            { visible: false },
+            'line3d',
+            ['point1', 'point2']
+        );
+
+        endpoints[0].F = function () {
+            var r = 0;
+            if (Type.evaluate(el.visProp.straightfirst)) {
+                r = -Infinity;
+            }
+            return el.getPointCoords(r);
+        };
+        endpoints[1].F = function () {
+            var r = 1;
+            if (Type.evaluate(el.visProp.straightlast)) {
+                r = Infinity;
+            }
+            return el.getPointCoords(r);
+        };
+        endpoints[0].prepareUpdate().update();
+        endpoints[1].prepareUpdate().update();
+
+        // The 2D line is always a segment.
+        attr = el.setAttr2D(attr);
+        attr.straightfirst = false;
+        attr.straightlast = false;
+        el.element2D = view.create('segment', [endpoints[0].element2D, endpoints[1].element2D], attr);
+        el.element2D.view = view;
+
+        /**
+         * Shadow points that determine the visible line.
+         * This is of relevance if the line is defined by two points and has straightFirst or straightLast set to true.
+         * In such a case, the shadow points are the intersection of the line with the cube.
+         *
+         * @name JXG.Point3D.endpoints
+         * @type Array
+         * @private
+         */
+        el.endpoints = endpoints;
+        el.addChild(endpoints[0]);
+        el.addChild(endpoints[1]);
+        el.setParents(endpoints);
+
     } else {
         // Line defined by point, direction and range
+
         point = Type.providePoints3D(view, [parents[1]], attributes, 'line3d', ['point'])[0];
 
         // Directions are handled as arrays of length 4,
@@ -270,33 +355,63 @@ JXG.createLine3D = function (board, parents, attributes) {
         el = new JXG.Line3D(view, point, direction, range, attr);
 
         // Now set the real points which define the line
-        /** @ignore */
+        /**
+         * @class
+         * @ignore
+         */
         points[0].F = function () {
             return el.getPointCoords(Type.evaluate(el.range[0]));
         };
         points[0].prepareUpdate().update();
         point1 = points[0];
 
-        /** @ignore */
+        /**
+         * @class
+         * @ignore
+         */
         points[1].F = function () {
             return el.getPointCoords(Type.evaluate(el.range[1]));
         };
         points[1].prepareUpdate().update();
         point2 = points[1];
+
+        attr = el.setAttr2D(attr);
+        attr.straightfirst = false;
+        attr.straightlast = false;
+        el.element2D = view.create('segment', [point1.element2D, point2.element2D], attr);
+        el.element2D.view = view;
+
+        el.endpoints = points;
     }
     // TODO Throw error
 
-    attr = el.setAttr2D(attr);
-    el.element2D = view.create('segment', [point1.element2D, point2.element2D], attr);
     el.addChild(el.element2D);
     el.inherits.push(el.element2D);
     el.element2D.setParents(el);
     // el.setParents([point1.id, point2.id]);
 
-    point1.addChild(el);
-    point2.addChild(el);
     el.point1 = point1;
     el.point2 = point2;
+    if (el.point1._is_new) {
+        el.addChild(el.point1);
+        delete el.point1._is_new;
+    } else {
+        el.point1.addChild(el);
+    }
+    if (el.point2._is_new) {
+        el.addChild(el.point2);
+        delete el.point2._is_new;
+    } else {
+        el.point2.addChild(el);
+    }
+    if (Type.exists(point)) {
+        if (point._is_new) {
+            el.addChild(point);
+            delete point._is_new;
+        } else {
+            point.addChild(el);
+        }
+    }
 
     el.update();
     el.element2D.prepareUpdate().update().updateRenderer();
@@ -315,7 +430,7 @@ JXG.registerElement('line3d', JXG.createLine3D);
  * @augments JXG.GeometryElement3D
  * @augments JXG.GeometryElement
  * @param {View3D} view
- * @param {Point3D,Array} point
+ * @param {Point3D|Array} point
  * @param {Array} direction1
  * @param {Array} range1
  * @param {Array} direction2
@@ -343,7 +458,7 @@ JXG.Plane3D = function (view, point, dir1, range1, dir2, range2, attributes) {
      * Two linearly independent vectors - together with a point - define the plane. Each of these direction vectors is an
      * array of numbers or functions (of length 3) or function returning array of length 3.
      *
-     * @type Array,Function
+     * @type Array|Function
      *
      * @see JXG.Plane3D#point
      * @see JXG.Plane3D#direction2
@@ -354,7 +469,7 @@ JXG.Plane3D = function (view, point, dir1, range1, dir2, range2, attributes) {
      * Two linearly independent vectors - together with a point - define the plane. Each of these direction vectors is an
      * array of numbers or functions (of length 3) or function returning array of length 3.
      *
-     * @type Array,Function
+     * @type Array|Function
      * @see JXG.Plane3D#point
      * @see JXG.Plane3D#direction1
      */
@@ -537,7 +652,7 @@ JXG.extend(
                 }
 
                 // Concatenate the intersection points to a polygon.
-                // If all wents well, each intersection should appear
+                // If all went well, each intersection should appear
                 // twice in the list.
                 first = 0;
                 pos = first;
@@ -658,8 +773,10 @@ JXG.createPlane3D = function (board, parents, attributes) {
 
     attr = el.setAttr2D(attr);
     el.element2D = view.create('curve', [[], []], attr);
+    el.element2D.view = view;
 
     /**
+     * @class
      * @ignore
      */
     el.element2D.updateDataArray = function () {
@@ -679,16 +796,18 @@ JXG.createPlane3D = function (board, parents, attributes) {
         Math.abs(el.range2[1]) !== Infinity
     ) {
         grid = view.create('mesh3d', [
-                function () {
-                    return point.coords;
-                },
-                dir1, range1, dir2, range2
-            ], attr
+            function () {
+                return point.coords;
+            },
+            dir1, range1, dir2, range2
+        ], attr
         );
         el.grid = grid;
         el.addChild(grid);
         el.inherits.push(grid);
         grid.setParents(el);
+        el.grid.view = view;
+
     }
 
     el.element2D.prepareUpdate().update();
@@ -698,4 +817,114 @@ JXG.createPlane3D = function (board, parents, attributes) {
 
     return el;
 };
+
 JXG.registerElement('plane3d', JXG.createPlane3D);
+
+/**
+ * @class An intersection line is a line which lives on two JSXGraph elements.
+ * The following element types can be (mutually) intersected: plane.
+ *
+ * @pseudo
+ * @name IntersectionLine3D
+ * @augments JXG.Line3D
+ * @constructor
+ * @type JXG.Line3D
+ * @throws {Exception} If the element cannot be constructed with the given parent objects an exception is thrown.
+ * @param {JXG.Plane3D_JXG.Plane3D} el1,el2 The result will be the intersection of el1 and el2.
+ * @example
+ * // Create the intersection line of two planes
+ * var view = board.create(
+ *     'view3d',
+ *     [[-6, -3], [8, 8],
+ *     [[0, 3], [0, 3], [0, 3]]],
+ *     {
+ *         xPlaneRear: {fillOpacity: 0.2, gradient: null},
+ *         yPlaneRear: {fillOpacity: 0.2, gradient: null},
+ *         zPlaneRear: {fillOpacity: 0.2, gradient: null}
+ *     }
+ * );
+ * var a = view.create('point3d', [0, 0, 0]);
+ *
+ * var p1 = view.create(
+ *    'plane3d',
+ *     [a, [1, 0, 0], [0, 1, 0]],
+ *     {fillColor: '#00ff80'}
+ * );
+ * var p2 = view.create(
+ *    'plane3d',
+ *     [a, [-2, 1, 1], [1, -2, 1]],
+ *     {fillColor: '#ff0000'}
+ * );
+ *
+ * var i = view.create('intersectionline3d', [p1, p2]);
+ *
+ * </pre><div id="JXGdb931076-b29a-4eff-b97e-4251aaf24943" class="jxgbox" style="width: 300px; height: 300px;"></div>
+ * <script type="text/javascript">
+ *     (function() {
+ *         var board = JXG.JSXGraph.initBoard('JXGdb931076-b29a-4eff-b97e-4251aaf24943',
+ *             {boundingbox: [-8, 8, 8,-8], axis: false, showcopyright: false, shownavigation: false});
+ *         var view = board.create(
+ *             'view3d',
+ *             [[-6, -3], [8, 8],
+ *             [[0, 3], [0, 3], [0, 3]]],
+ *             {
+ *                 xPlaneRear: {fillOpacity: 0.2, gradient: null},
+ *                 yPlaneRear: {fillOpacity: 0.2, gradient: null},
+ *                 zPlaneRear: {fillOpacity: 0.2, gradient: null}
+ *             }
+ *         );
+ *     var a = view.create('point3d', [0, 0, 0]);
+ *
+ *     var p1 = view.create(
+ *        'plane3d',
+ *         [a, [1, 0, 0], [0, 1, 0]],
+ *         {fillColor: '#00ff80'}
+ *     );
+ *     var p2 = view.create(
+ *        'plane3d',
+ *         [a, [-2, 1, 1], [1, -2, 1]],
+ *         {fillColor: '#ff0000'}
+ *     );
+ *
+ *     var i = view.create('intersectionline3d', [p1, p2]);
+ *
+ *     })();
+ *
+ * </script><pre>
+ *
+ */
+JXG.createIntersectionLine3D = function (board, parents, attributes) {
+    var view = parents[0],
+        el1 = parents[1],
+        el2 = parents[2],
+        ixnLine, i, func,
+        attr = Type.copyAttributes(attributes, board.options, "intersectionline3d"),
+        pts = [];
+
+    for (i = 0; i < 2; i++) {
+        func = Geometry.intersectionFunction3D(view, el1, el2, i);
+        pts[i] = view.create('point3d', func, attr['point' + (i + 1)]);
+    }
+    ixnLine = view.create('line3d', pts, attr);
+
+    try {
+        el1.addChild(ixnLine);
+        el2.addChild(ixnLine);
+    } catch (_e) {
+        throw new Error(
+            "JSXGraph: Can't create 'intersection' with parent types '" +
+            typeof parents[0] +
+            "' and '" +
+            typeof parents[1] +
+            "'."
+        );
+    }
+
+    ixnLine.type = Const.OBJECT_TYPE_INTERSECTION_LINE3D;
+    ixnLine.elType = 'intersectionline3d';
+    ixnLine.setParents([el1.id, el2.id]);
+
+    return ixnLine;
+};
+
+JXG.registerElement('intersectionline3d', JXG.createIntersectionLine3D);
