@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2023
+    Copyright 2008-2024
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -35,14 +35,14 @@
  * This object provides the coordinate handling of points, images and texts.
  */
 
-import JXG from "../jxg";
-import Mat from "../math/math";
-import Geometry from "../math/geometry";
-import Numerics from "../math/numerics";
-import Statistics from "../math/statistics";
-import Coords from "./coords";
-import Const from "./constants";
-import Type from "../utils/type";
+import JXG from "../jxg.js";
+import Mat from "../math/math.js";
+import Geometry from "../math/geometry.js";
+import Numerics from "../math/numerics.js";
+import Statistics from "../math/statistics.js";
+import Coords from "./coords.js";
+import Const from "./constants.js";
+import Type from "../utils/type.js";
 
 /**
  * An element containing coords is the basic geometric element. Based on points lines and circles can be constructed which can be intersected
@@ -153,12 +153,16 @@ JXG.CoordsElement = function (coordinates, isLabel) {
         makeIntersection: "makeIntersection",
         X: "X",
         Y: "Y",
+        Coords: "Coords",
         free: "free",
         setPosition: "setGliderPosition",
         setGliderPosition: "setGliderPosition",
         addConstraint: "addConstraint",
         dist: "Dist",
-        onPolygon: "onPolygon"
+        Dist: "Dist",
+        onPolygon: "onPolygon",
+        startAnimation: "startAnimation",
+        stopAnimation: "stopAnimation"
     });
 
     /*
@@ -245,6 +249,7 @@ JXG.extend(
                 newCoords, newPos,
                 doRound = false,
                 ev_sw,
+                snappedTo, snapValues,
                 slide = this.slideObject,
                 res, cu,
                 slides = [],
@@ -373,20 +378,28 @@ JXG.extend(
                     }
                 }
 
-                // Snap the glider point of the slider into its appropiate position
-                // First, recalculate the new value of this.position
-                // Second, call update(fromParent==true) to make the positioning snappier.
-                ev_sw = Type.evaluate(this.visProp.snapwidth);
-                if (
-                    Type.evaluate(ev_sw) > 0.0 &&
-                    Math.abs(this._smax - this._smin) >= Mat.eps
-                ) {
-                    newPos = Math.max(Math.min(newPos, 1), 0);
-
-                    v = newPos * (this._smax - this._smin) + this._smin;
-                    v = Math.round(v / ev_sw) * ev_sw;
-                    newPos = (v - this._smin) / (this._smax - this._smin);
+                // Snap the glider to snap values.
+                snappedTo = this.findClosestSnapValue(newPos);
+                if(snappedTo !== null) {
+                    snapValues = Type.evaluate(this.visProp.snapvalues);
+                    newPos = (snapValues[snappedTo] - this._smin) / (this._smax - this._smin);
                     this.update(true);
+                } else {
+                    // Snap the glider point of the slider into its appropiate position
+                    // First, recalculate the new value of this.position
+                    // Second, call update(fromParent==true) to make the positioning snappier.
+                    ev_sw = Type.evaluate(this.visProp.snapwidth);
+                    if (
+                        Type.evaluate(ev_sw) > 0.0 &&
+                        Math.abs(this._smax - this._smin) >= Mat.eps
+                    ) {
+                        newPos = Math.max(Math.min(newPos, 1), 0);
+
+                        v = newPos * (this._smax - this._smin) + this._smin;
+                        v = Math.round(v / ev_sw) * ev_sw;
+                        newPos = (v - this._smin) / (this._smax - this._smin);
+                        this.update(true);
+                    }
                 }
 
                 p1c = slide.point1.coords;
@@ -533,6 +546,37 @@ JXG.extend(
             this.position = newPos;
         },
 
+        /**
+         * Find the closest entry in snapValues that is within snapValueDistance of pos.
+         *
+         * @param {Number} pos Value for which snapping is calculated.
+         * @returns {Number} Index of the value to snap to, or null.
+         * @private
+         */
+        findClosestSnapValue: function(pos) {
+            var i, d,
+                snapValues, snapValueDistance,
+                snappedTo = null;
+
+            // Snap the glider to snap values.
+            snapValues = Type.evaluate(this.visProp.snapvalues);
+            snapValueDistance = Type.evaluate(this.visProp.snapvaluedistance);
+
+            if (Type.isArray(snapValues) &&
+                Math.abs(this._smax - this._smin) >= Mat.eps &&
+                snapValueDistance > 0.0) {
+                for (i = 0; i < snapValues.length; i++) {
+                    d = Math.abs(pos * (this._smax - this._smin) + this._smin - snapValues[i]);
+                    if (d < snapValueDistance) {
+                        snapValueDistance = d;
+                        snappedTo = i;
+                    }
+                }
+            }
+
+            return snappedTo;
+        },
+
         /**
          * Update of a glider in case a parent element has been updated. That means the
          * relative position of the glider stays the same.
@@ -805,6 +849,70 @@ JXG.extend(
             return this.coords.usrCoords[0];
         },
 
+        /**
+         * Getter method for coordinates x, y and (optional) z.
+         * @param {Number|String} [digits='auto'] Truncating rule for the digits in the infobox.
+         * <ul>
+         * <li>'auto': done automatically by JXG.autoDigits()
+         * <li>'none': no truncation
+         * <li>number: truncate after "number digits" with JXG.toFixed()
+         * </ul>
+         * @param {Boolean} [withZ=false] If set to true the return value will be <tt>(x | y | z)</tt> instead of <tt>(x, y)</tt>.
+         * @returns {String} User coordinates of point.
+         */
+        Coords: function(withZ) {
+            if (withZ) {
+                return this.coords.usrCoords.slice();
+            }
+            return this.coords.usrCoords.slice(1);
+        },
+        // Coords: function (digits, withZ) {
+        //     var arr, sep;
+
+        //     digits = digits || 'auto';
+
+        //     if (withZ) {
+        //         sep = ' | ';
+        //     } else {
+        //         sep = ', ';
+        //     }
+
+        //     if (digits === 'none') {
+        //         arr = [this.X(), sep, this.Y()];
+        //         if (withZ) {
+        //             arr.push(sep, this.Z());
+        //         }
+
+        //     } else if (digits === 'auto') {
+        //         if (this.useLocale()) {
+        //             arr = [this.formatNumberLocale(this.X()), sep, this.formatNumberLocale(this.Y())];
+        //             if (withZ) {
+        //                 arr.push(sep, this.formatNumberLocale(this.Z()));
+        //             }
+        //         } else {
+        //             arr = [Type.autoDigits(this.X()), sep, Type.autoDigits(this.Y())];
+        //             if (withZ) {
+        //                 arr.push(sep, Type.autoDigits(this.Z()));
+        //             }
+        //         }
+
+        //     } else {
+        //         if (this.useLocale()) {
+        //             arr = [this.formatNumberLocale(this.X(), digits), sep, this.formatNumberLocale(this.Y(), digits)];
+        //             if (withZ) {
+        //                 arr.push(sep, this.formatNumberLocale(this.Z(), digits));
+        //             }
+        //         } else {
+        //             arr = [Type.toFixed(this.X(), digits), sep, Type.toFixed(this.Y(), digits)];
+        //             if (withZ) {
+        //                 arr.push(sep, Type.toFixed(this.Z(), digits));
+        //             }
+        //         }
+        //     }
+
+        //     return '(' + arr.join('') + ')';
+        // },
+
         /**
          * New evaluation of the function term.
          * This is required for CAS-points: Their XTerm() method is
@@ -843,7 +951,7 @@ JXG.extend(
 
         /**
          * Getter method for the distance to a second point, this is required for CAS-elements.
-         * Here, function inlining seems to be worthwile  (for plotting).
+         * Here, function inlining seems to be worthwile (for plotting).
          * @param {JXG.Point} point2 The point to which the distance shall be calculated.
          * @returns {Number} Distance in user coordinate to the given point
          */
@@ -856,7 +964,7 @@ JXG.extend(
 
         /**
          * Alias for {@link JXG.Element#handleSnapToGrid}
-         * @param {Boolean} force force snapping independent from what the snaptogrid attribute says
+         * @param {Boolean} force force snapping independent of what the snaptogrid attribute says
          * @returns {JXG.CoordsElement} Reference to this element
          */
         snapToGrid: function (force) {
@@ -868,7 +976,7 @@ JXG.extend(
          * {@link JXG.Point#attractorDistance}.
          * The function uses the coords object of the point as
          * its actual position.
-         * @param {Boolean} force force snapping independent from what the snaptogrid attribute says
+         * @param {Boolean} force force snapping independent of what the snaptogrid attribute says
          * @returns {JXG.Point} Reference to this element
          */
         handleSnapToPoints: function (force) {
@@ -938,7 +1046,7 @@ JXG.extend(
         /**
          * Alias for {@link JXG.CoordsElement#handleSnapToPoints}.
          *
-         * @param {Boolean} force force snapping independent from what the snaptogrid attribute says
+         * @param {Boolean} force force snapping independent of what the snaptogrid attribute says
          * @returns {JXG.Point} Reference to this element
          */
         snapToPoints: function (force) {
@@ -1231,8 +1339,8 @@ JXG.extend(
                 // It may not be sufficient to remove the point from
                 // the list of childElement. For complex dependencies
                 // one may have to go to the list of ancestor and descendants.  A.W.
-                // Yes indeed, see #51 on github bugtracker
-                //  delete this.slideObject.childElements[this.id];
+                // Yes indeed, see #51 on github bug tracker
+                //   delete this.slideObject.childElements[this.id];
                 this.slideObject.removeChild(this);
 
                 if (this.slideObjects.length === 0) {
@@ -1321,6 +1429,7 @@ JXG.extend(
 
             // A free point does not depend on anything. Remove all ancestors.
             this.ancestors = {}; // only remove the reference
+            this.parents = [];
 
             // Completely remove all slideObjects of the element
             this.slideObject = null;
@@ -1584,10 +1693,10 @@ JXG.extend(
 
         /**
          * Animate the point.
-         * @param {Number,Function} direction The direction the glider is animated. Can be +1 or -1.
-         * @param {Number,Function} stepCount The number of steps in which the parent element is divided.
+         * @param {Number|Function} direction The direction the glider is animated. Can be +1 or -1.
+         * @param {Number|Function} stepCount The number of steps in which the parent element is divided.
          * Must be at least 1.
-         * @param {Number,Function} delay Time in msec between two animation steps. Default is 250.
+         * @param {Number|Function} delay Time in msec between two animation steps. Default is 250.
          * @returns {JXG.CoordsElement} Reference to iself.
          *
          * @name Glider#startAnimation
@@ -1918,7 +2027,7 @@ JXG.extend(
          * @returns {JXG.CoordsElement} Reference to itself.
          */
         _anim: function (direction, stepCount) {
-            var dX, dY, alpha, startPoint, newX, radius, sp1c, sp2c, res, d;
+            var dX, dY, alpha, startPoint, newX, radius, sp1c, sp2c, res;
 
             this.intervalCount += 1;
             if (this.intervalCount > stepCount) {
