@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2023
+    Copyright 2008-2024
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -36,16 +36,20 @@
  * @fileoverview In this file the Text element is defined.
  */
 
-import JXG from "../jxg";
-import Const from "./constants";
-import GeometryElement from "./element";
-import GeonextParser from "../parser/geonext";
-import Env from "../utils/env";
-import Type from "../utils/type";
-import Mat from "../math/math";
-import CoordsElement from "./coordselement";
+import JXG from "../jxg.js";
+import Const from "./constants.js";
+import GeometryElement from "./element.js";
+import GeonextParser from "../parser/geonext.js";
+import Env from "../utils/env.js";
+import Type from "../utils/type.js";
+import Mat from "../math/math.js";
+import CoordsElement from "./coordselement.js";
 
 var priv = {
+    /**
+     * @class
+     * @ignore
+     */
     HTMLSliderInputEventHandler: function () {
         this._val = parseFloat(this.rendNodeRange.value);
         this.rendNodeOut.value = this.rendNodeRange.value;
@@ -119,7 +123,9 @@ JXG.Text = function (board, coords, attributes, content) {
     this.methodMap = Type.deepCopy(this.methodMap, {
         setText: "setTextJessieCode",
         // free: 'free',
-        // move: "setCoords"
+        move: "setCoords",
+        Size: "getSize",
+        setAutoPosition: "setAutoPosition"
     });
 };
 
@@ -131,15 +137,15 @@ JXG.extend(
     /** @lends JXG.Text.prototype */ {
         /**
          * @private
-         * Test if the screen coordinates (x,y) are in a small stripe
-         * at the left side or at the right side of the text.
-         * Sensitivity is set in this.board.options.precision.hasPoint.
-         * If dragarea is set to 'all' (default), tests if the screen
-         * coordinates (x,y) are in within the text boundary.
          * @param {Number} x
          * @param {Number} y
          * @returns {Boolean}
-         */
+        */
+        // Test if the screen coordinates (x,y) are in a small stripe
+        // at the left side or at the right side of the text.
+        // Sensitivity is set in this.board.options.precision.hasPoint.
+        // If dragarea is set to 'all' (default), tests if the screen
+        // coordinates (x,y) are in within the text boundary.
         hasPoint: function (x, y) {
             var lft, rt, top, bot, ax, ay, type, r;
 
@@ -234,12 +240,20 @@ JXG.extend(
                     }
                 };
             } else {
-                if (Type.isNumber(text)) {
-                    digits = Type.evaluate(this.visProp.digits);
-                    if (this.useLocale()) {
-                        this.content = this.formatNumberLocale(text, digits);
+                if (Type.isNumber(text) && Type.evaluate(this.visProp.formatnumber)) {
+                    if (Type.evaluate(this.visProp.tofraction)) {
+                        if (ev_um) {
+                            this.content = '\\(' + Type.toFraction(text, true) + '\\)';
+                        } else {
+                            this.content = Type.toFraction(text, ev_uk);
+                        }
                     } else {
-                        this.content = Type.toFixed(text, digits);
+                        digits = Type.evaluate(this.visProp.digits);
+                        if (this.useLocale()) {
+                            this.content = this.formatNumberLocale(text, digits);
+                        } else {
+                            this.content = Type.toFixed(text, digits);
+                        }
                     }
                 } else if (Type.isString(text) && ev_p) {
                     if (Type.evaluate(this.visProp.useasciimathml)) {
@@ -307,7 +321,10 @@ JXG.extend(
                                     }
                                 } else {
                                     t = that.content[i];
-                                    if (t.at(0) === '"' && t.at(-1) === '"') {
+                                    // Instead of 't.at(t.length - 1)' also 't.(-1)' should work.
+                                    // However in Moodle 4.2 't.(-1)' returns an empty string.
+                                    // In plain HTML pages it works.
+                                    if (t[0] === '"' && t[t.length - 1] === '"') {
                                         t = t.slice(1, -1);
                                     }
                                 }
@@ -406,7 +423,7 @@ JXG.extend(
          * JSXGraph needs {@link JXG.Text#size} for applying rotations in IE and
          * for aligning text.
          *
-         * @return {[type]} [description]
+         * @return {this} [description]
          */
         updateSize: function () {
             var tmp,
@@ -498,22 +515,21 @@ JXG.extend(
             var j,
                 i = te.indexOf("_{");
 
-            // the regexp in here are not used for filtering but to provide some kind of sugar for label creation,
+            // The regexp in here are not used for filtering but to provide some kind of sugar for label creation,
             // i.e. replacing _{...} with <sub>...</sub>. What is passed would get out anyway.
             /*jslint regexp: true*/
-
             while (i >= 0) {
-                te = te.substr(0, i) + te.substr(i).replace(/_\{/, "<sub>");
-                j = te.substr(i).indexOf("}");
+                te = te.slice(0, i) + te.slice(i).replace(/_\{/, "<sub>");
+                j = te.indexOf("}", i + 4);
                 if (j >= 0) {
-                    te = te.substr(0, j) + te.substr(j).replace(/\}/, "</sub>");
+                    te = te.slice(0, j) + te.slice(j).replace(/\}/, "</sub>");
                 }
                 i = te.indexOf("_{");
             }
 
             i = te.indexOf("_");
             while (i >= 0) {
-                te = te.substr(0, i) + te.substr(i).replace(/_(.?)/, "<sub>$1</sub>");
+                te = te.slice(0, i) + te.slice(i).replace(/_(.?)/, "<sub>$1</sub>");
                 i = te.indexOf("_");
             }
 
@@ -533,22 +549,21 @@ JXG.extend(
             var j,
                 i = te.indexOf("^{");
 
-            // the regexp in here are not used for filtering but to provide some kind of sugar for label creation,
+            // The regexp in here are not used for filtering but to provide some kind of sugar for label creation,
             // i.e. replacing ^{...} with <sup>...</sup>. What is passed would get out anyway.
             /*jslint regexp: true*/
-
             while (i >= 0) {
-                te = te.substr(0, i) + te.substr(i).replace(/\^\{/, "<sup>");
-                j = te.substr(i).indexOf("}");
+                te = te.slice(0, i) + te.slice(i).replace(/\^\{/, "<sup>");
+                j = te.indexOf("}", i + 4);
                 if (j >= 0) {
-                    te = te.substr(0, j) + te.substr(j).replace(/\}/, "</sup>");
+                    te = te.slice(0, j) + te.slice(j).replace(/\}/, "</sup>");
                 }
                 i = te.indexOf("^{");
             }
 
             i = te.indexOf("^");
             while (i >= 0) {
-                te = te.substr(0, i) + te.substr(i).replace(/\^(.?)/, "<sup>$1</sup>");
+                te = te.slice(0, i) + te.slice(i).replace(/\^(.?)/, "<sup>$1</sup>");
                 i = te.indexOf("^");
             }
 
@@ -654,7 +669,7 @@ JXG.extend(
         },
 
         /**
-         * The update function of the renderert
+         * The update function of the renderer
          * is called.
          * @private
          */
@@ -692,7 +707,7 @@ JXG.extend(
          * @param{String} contentStr String to be parsed
          * @param{Boolean} [expand] Optional flag if shortened math syntax is allowed (e.g. 3x instead of 3*x).
          * @param{Boolean} [avoidGeonext2JS] Optional flag if geonext2JS should be called. For backwards compatibility
-         * this has to be set explicitely to true.
+         * this has to be set explicitly to true.
          * @param{Boolean} [outputTeX] Optional flag which has to be true if the resulting term will be sent to MathJax or KaTeX.
          * If true, "_" and "^" are NOT replaced by HTML tags sub and sup. Default: false, i.e. the replacement is done.
          * This flag allows the combination of &lt;value&gt; tag containing calculations with TeX output.
@@ -1000,7 +1015,7 @@ JXG.extend(
 
                 if (res !== null) {
                     GeonextParser.findDependencies(this, res[1], this.board);
-                    content = content.substr(res.index);
+                    content = content.slice(res.index);
                     content = content.replace(search, "");
                 }
             } while (res !== null);
@@ -1124,12 +1139,11 @@ JXG.extend(
          * @param  {Number} h width of the box in pixel
          * @return {Number}   Number of overlapping elements
          */
-        getNumberofConflicts: function (x, y, w, h) {
+        getNumberOfConflicts: function (x, y, w, h) {
             var count = 0,
-                i,
-                obj,
-                le,
-                savePointPrecision;
+                i, obj, le,
+                savePointPrecision,
+                saveHasInnerPoints;
 
             // Set the precision of hasPoint to half the max if label isn't too long
             savePointPrecision = this.board.options.precision.hasPoint;
@@ -1139,6 +1153,8 @@ JXG.extend(
             // Make it compatible with the objects' visProp.precision attribute
             for (i = 0, le = this.board.objectsList.length; i < le; i++) {
                 obj = this.board.objectsList[i];
+                saveHasInnerPoints = obj.visProp.hasinnerpoints;
+                obj.visProp.hasinnerpoints = false;
                 if (
                     obj.visPropCalc.visible &&
                     obj.elType !== "axis" &&
@@ -1149,6 +1165,7 @@ JXG.extend(
                 ) {
                     count++;
                 }
+                obj.visProp.hasinnerpoints = saveHasInnerPoints;
             }
             this.board.options.precision.hasPoint = savePointPrecision;
 
@@ -1163,33 +1180,22 @@ JXG.extend(
          * @returns {JXG.Text} Reference to the text object.
          */
         setAutoPosition: function () {
-            var x,
-                y,
-                cx,
-                cy,
+            var x, y, cx, cy,
                 anchorCoords,
                 // anchorX, anchorY,
                 w = this.size[0],
                 h = this.size[1],
-                start_angle,
-                angle,
+                start_angle, angle,
                 optimum = {
                     conflicts: Infinity,
                     angle: 0,
                     r: 0
                 },
-                max_r,
-                delta_r,
-                conflicts,
-                offset,
-                r,
+                max_r, delta_r,
+                conflicts, offset, r,
                 num_positions = 12,
                 step = (2 * Math.PI) / num_positions,
-                j,
-                dx,
-                dy,
-                co,
-                si;
+                j, dx, dy, co, si;
 
             if (
                 this === this.board.infobox ||
@@ -1212,15 +1218,15 @@ JXG.extend(
             dx = offset[0];
             dy = offset[1];
 
-            conflicts = this.getNumberofConflicts(cx + dx, cy - dy, w, h);
+            conflicts = this.getNumberOfConflicts(cx + dx, cy - dy, w, h);
             if (conflicts === 0) {
                 return this;
             }
             // console.log(this.id, conflicts, w, h);
             // r = Geometry.distance([0, 0], offset, 2);
 
-            r = 12;
-            max_r = 28;
+            r = Type.evaluate(this.visProp.autopositionmindistance);
+            max_r = Type.evaluate(this.visProp.autopositionmaxdistance);
             delta_r = 0.2 * r;
 
             start_angle = Math.atan2(dy, dx);
@@ -1229,7 +1235,7 @@ JXG.extend(
             optimum.angle = start_angle;
             optimum.r = r;
 
-            while (optimum.conflicts > 0 && r < max_r) {
+            while (optimum.conflicts > 0 && r <= max_r) {
                 for (
                     j = 1, angle = start_angle + step;
                     j < num_positions && optimum.conflicts > 0;
@@ -1241,7 +1247,7 @@ JXG.extend(
                     x = cx + r * co;
                     y = cy - r * si;
 
-                    conflicts = this.getNumberofConflicts(x, y, w, h);
+                    conflicts = this.getNumberOfConflicts(x, y, w, h);
                     if (conflicts < optimum.conflicts) {
                         optimum.conflicts = conflicts;
                         optimum.angle = angle;
@@ -1289,7 +1295,6 @@ JXG.extend(
  * If HTML should be displayed in an inbetween layer, conder to use an element of type {@link ForeignObject} (available in svg renderer, only).
  * </ul>
  * @pseudo
- * @description
  * @name Text
  * @augments JXG.Text
  * @constructor
@@ -1389,7 +1394,6 @@ JXG.registerElement("text", JXG.createText);
  * Labels are handled internally by JSXGraph, only. There is NO constructor "board.create('label', ...)".
  *
  * @pseudo
- * @description
  * @name Label
  * @augments JXG.Text
  * @constructor
