@@ -1,85 +1,93 @@
 /* eslint-disable one-var */
-import JXG from "./jxg";
-import Env from "./utils/env"; // Needed below
-import "./base/constants";
-import "./utils/type";
-import "./utils/xml";
-import "./utils/event";
-import "./utils/expect";
-import "./math/math";
-import "./math/probfuncs";
-import "./math/ia";
-import "./math/extrapolate";
-import "./math/qdt";
-import "./math/numerics";
-import "./math/nlp";
-import "./math/plot";
-import "./math/metapost";
-import "./math/statistics";
-import "./math/symbolic";
-import "./math/geometry";
-import "./math/clip";
-import "./math/poly";
-import "./math/complex";
-import "./renderer/abstract";
-import "./reader/file";
-import "./parser/geonext";
-import "./base/board";
-import "./options";
-import "./jsxgraph";
-import "./base/element";
-import "./base/coords";
-import "./base/coordselement";
-import "./base/point";
-import "./base/line";
-import "./base/group";
-import "./base/circle";
-import "./element/conic";
-import "./base/polygon";
-import "./base/curve";
-import "./element/arc";
-import "./element/sector";
-import "./base/composition";
-import "./element/composition";
-import "./element/locus";
-import "./base/text";
-import "./base/image";
-import "./element/slider";
-import "./element/measure";
-import "./base/chart";
-import "./base/transformation";
-import "./base/turtle";
-import "./utils/color";
-import "./base/ticks";
-import "./utils/zip";
-import "./utils/base64";
-import "./utils/uuid";
-import "./utils/encoding";
-import "./server/server";
-import "./parser/datasource";
-import "./parser/jessiecode";
-import "./parser/ca";
-import "./utils/dump";
-import "./renderer/svg";
-import "./renderer/vml";
-import "./renderer/canvas";
-import "./renderer/no";
-import "./element/comb";
-import "./element/slopetriangle";
-import "./element/checkbox";
-import "./element/input";
-import "./element/button";
-import "./element/vectorfield";
-import "./element/smartlabel";
-import "./base/foreignobject";
-import "./options3d";
-import "./3d/view3d";
-import "./3d/element3d";
-import "./3d/box3d";
-import "./3d/point3d";
-import "./3d/curve3d";
-import "./3d/linspace3d";
-import "./3d/surface3d";
+import JXG from './jxg.js';
+import Env from './utils/env.js'; // Needed below
+import './base/constants.js';
+import './utils/type.js';
+import './utils/xml.js';
+import './utils/event.js';
+import './utils/expect.js';
+import './math/math.js';
+import './math/probfuncs.js';
+import './math/ia.js';
+import './math/extrapolate.js';
+import './math/qdt.js';
+import './math/bqdt.js';
+import './math/numerics.js';
+import './math/nlp.js';
+import './math/plot.js';
+import './math/implicitplot.js';
+import './math/metapost.js';
+import './math/statistics.js';
+import './math/symbolic.js';
+import './math/geometry.js';
+import './math/clip.js';
+import './math/poly.js';
+import './math/complex.js';
+import './renderer/abstract.js';
+import './reader/file.js';
+import './parser/geonext.js';
+import './base/board.js';
+import './options.js';
+import './jsxgraph.js';
+import './base/element.js';
+import './base/coords.js';
+import './base/coordselement.js';
+import './base/point.js';
+import './base/line.js';
+import './base/group.js';
+import './base/circle.js';
+import './element/conic.js';
+import './base/polygon.js';
+import './base/curve.js';
+import './element/arc.js';
+import './element/sector.js';
+import './base/composition.js';
+import './element/composition.js';
+import './element/grid.js';
+import './element/locus.js';
+import './base/text.js';
+import './base/image.js';
+import './element/slider.js';
+import './element/measure.js';
+import './base/chart.js';
+import './base/transformation.js';
+import './base/turtle.js';
+import './utils/color.js';
+import './base/ticks.js';
+import './utils/zip.js';
+import './utils/base64.js';
+import './utils/uuid.js';
+import './utils/encoding.js';
+import './server/server.js';
+import './parser/datasource.js';
+import './parser/jessiecode.js';
+import './parser/prefix.js';
+import './parser/ca.js';
+import './utils/dump.js';
+import './renderer/svg.js';
+import './renderer/vml.js';
+import './renderer/canvas.js';
+import './renderer/no.js';
+import './element/comb.js';
+import './element/slopetriangle.js';
+import './element/checkbox.js';
+import './element/input.js';
+import './element/button.js';
+import './element/vectorfield.js';
+import './element/smartlabel.js';
+import './base/foreignobject.js';
+import './options3d.js';
+import './3d/view3d.js';
+import './3d/element3d.js';
+import './3d/box3d.js';
+import './3d/circle3d.js';
+import './3d/point3d.js';
+import './3d/curve3d.js';
+import './3d/linspace3d.js';
+import './3d/polygon3d.js';
+import './3d/sphere3d.js';
+import './3d/surface3d.js';
+import './themes/mono_thin.js';
 
 // The following exports are used to restore granular objects.
 // This is consistent with 1.4.x when a UMD bundle is used with a SystemJS loader.
@@ -90,6 +98,7 @@ import "./3d/surface3d";
 export const COORDS_BY_SCREEN = JXG.COORDS_BY_SCREEN;
 export const COORDS_BY_USER = JXG.COORDS_BY_USER;
 export const Dump = JXG.Dump;
+export const Expect = JXG.Expect;
 export const JSXGraph = JXG.JSXGraph;
 export const Mat = JXG.Math;
 export const Options = JXG.Options;
@@ -111,6 +120,7 @@ export const GeometryElement = JXG.GeometryElement;
 export const Group = JXG.Group;
 export const Image = JXG.Image;
 export const JessieCode = JXG.JessieCode;
+export const Prefix = JXG.PrefixParser;
 export const Line = JXG.Line;
 export const Point = JXG.Point;
 export const Polygon = JXG.Polygon;
@@ -136,6 +146,7 @@ export const copyAttributes = JXG.copyAttributes;
 export const createEvalFunction = JXG.createEvalFunction;
 export const createFunction = JXG.createFunction;
 export const createHTMLSlider = JXG.createHTMLSlider;
+export const darkenColor = JXG.darkenColor;
 export const debug = JXG.debug;
 export const debugInt = JXG.debugInt;
 export const debugLine = JXG.debugLine;
@@ -160,16 +171,21 @@ export const hsv2rgb = JXG.hsv2rgb;
 export const isAndroid = JXG.isAndroid;
 export const isApple = JXG.isApple;
 export const isArray = JXG.isArray;
+export const isDesktop = JXG.isDesktop;
 export const isInArray = JXG.isInArray;
 export const isInObject = JXG.isInObject;
 export const isMetroApp = JXG.isMetroApp;
+export const isMobile = JXG.isMobile;
 export const isMozilla = JXG.isMozilla;
 export const isBoard = JXG.isBoard;
 export const isName = JXG.isName;
 export const isNode = JXG.isNode;
 export const isNumber = JXG.isNumber;
 export const isObject = JXG.isObject;
+export const isPoint = JXG.isPoint;
+export const isPoint3D = JXG.isPoint3D;
 export const isPointType = JXG.isPointType;
+export const isPointType3D = JXG.isPointType3D;
 export const isString = JXG.isString;
 export const isTouchDevice = JXG.isTouchDevice;
 export const isTransformationOrArray = JXG.isTransformationOrArray;
@@ -177,6 +193,7 @@ export const isWebWorker = JXG.isWebWorker;
 export const isWebkitAndroid = JXG.isWebkitAndroid;
 export const isWebkitApple = JXG.isWebkitApple;
 export const keys = JXG.keys;
+export const lightenColor = JXG.lightenColor;
 export const merge = JXG.merge;
 export const normalizePointFace = JXG.normalizePointFace;
 export const providePoints = JXG.providePoints;
@@ -226,7 +243,7 @@ if (Env.isBrowser) {
     // Nodejs compatibility is handled by webpack
     // OLD: in case 2) module is undefined, the export is set in src/jsxgraphnode.js using
     // the return value of this factory function
-    // } else if (Env.isNode() && typeof module === "object") {
+    // } else if (Env.isNode() && typeof module === 'object') {
     //     module.exports = JXG;
 } else if (Env.isWebWorker()) {
     self.JXG = JXG;
