@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2023
+    Copyright 2008-2024
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -36,20 +36,20 @@
  * @fileoverview The JSXGraph object is defined in this file. JXG.JSXGraph controls all boards.
  * It has methods to create, save, load and free boards. Additionally some helper functions are
  * defined in this file directly in the JXG namespace.
- * @version 0.99
+ *
  */
 
-import JXG from "./jxg";
-import Env from "./utils/env";
-import Type from "./utils/type";
-import Mat from "./math/math";
-import Board from "./base/board";
-import FileReader from "./reader/file";
-import Options from "./options";
-import SVGRenderer from "./renderer/svg";
-import VMLRenderer from "./renderer/vml";
-import CanvasRenderer from "./renderer/canvas";
-import NoRenderer from "./renderer/no";
+import JXG from "./jxg.js";
+import Env from "./utils/env.js";
+import Type from "./utils/type.js";
+// import Mat from "./math/math.js";
+import Board from "./base/board.js";
+import FileReader from "./reader/file.js";
+import Options from "./options.js";
+import SVGRenderer from "./renderer/svg.js";
+import VMLRenderer from "./renderer/vml.js";
+import CanvasRenderer from "./renderer/canvas.js";
+import NoRenderer from "./renderer/no.js";
 
 /**
  * Constructs a new JSXGraph singleton object.
@@ -115,7 +115,7 @@ JXG.JSXGraph = {
     /**
      * Initialize the rendering engine
      *
-     * @param  {String} box        HTML id of the div-element which hosts the JSXGraph construction
+     * @param  {String} box        id of or reference to the div element which hosts the JSXGraph construction
      * @param  {Object} dim        The dimensions of the board
      * @param  {Object} doc        Usually, this is document object of the browser window.  If false or null, this defaults
      * to the document object of the browser.
@@ -134,7 +134,7 @@ JXG.JSXGraph = {
         }
 
         if (typeof doc === "object" && box !== null) {
-            boxid = doc.getElementById(box);
+            boxid = (Type.isString(box)) ? doc.getElementById(box) : box;
 
             // Remove everything from the container before initializing the renderer and the board
             while (boxid.firstChild) {
@@ -170,9 +170,9 @@ JXG.JSXGraph = {
      *
      * @private
      */
-    _setAttributes: function (attributes) {
+    _setAttributes: function (attributes, options) {
         // merge attributes
-        var attr = Type.copyAttributes(attributes, Options, 'board'),
+        var attr = Type.copyAttributes(attributes, options, 'board'),
 
             // These attributes - which are objects - have to be copied separately.
             list = [
@@ -187,13 +187,13 @@ JXG.JSXGraph = {
 
         for (i = 0; i < len; i++) {
             key = list[i];
-            attr[key] = Type.copyAttributes(attr, Options, 'board', key);
+            attr[key] = Type.copyAttributes(attr, options, 'board', key);
         }
 
         // Treat moveTarget separately, because deepCopy will not work here.
         // Reason: moveTarget will be an HTML node and it is prevented that Type.deepCopy will copy it.
         attr.movetarget =
-            attributes.moveTarget || attributes.movetarget || Options.board.moveTarget;
+            attributes.moveTarget || attributes.movetarget || options.board.moveTarget;
 
         return attr;
     },
@@ -218,7 +218,7 @@ JXG.JSXGraph = {
 
     /**
      *
-     * @param {String} container HTML-ID to the HTML-element in which the board is painted.
+     * @param {String|Object} container id of or reference to the HTML element in which the board is painted.
      * @param {Object} attr An object that sets some of the board properties.
      *
      * @private
@@ -239,7 +239,7 @@ JXG.JSXGraph = {
                 doc = document;
             }
 
-        node_jsx = doc.getElementById(container);
+        node_jsx = (Type.isString(container)) ? doc.getElementById(container) : container;
         doc_glob = node_jsx.ownerDocument; // This is the window.document element, needed below.
         parent = node_jsx.parentNode;
 
@@ -291,7 +291,7 @@ JXG.JSXGraph = {
 
     /**
      * Initialize a new board.
-     * @param {String} box HTML-ID to the HTML-element in which the board is painted.
+     * @param {String|Object} box id of or reference to the HTML element in which the board is painted.
      * @param {Object} attributes An object that sets some of the board properties. Most of these properties can be set via JXG.Options.
      * @param {Array} [attributes.boundingbox=[-5, 5, 5, -5]] An array containing four numbers describing the left, top, right and bottom boundary of the board in user coordinates
      * @param {Boolean} [attributes.keepaspectratio=false] If <tt>true</tt>, the bounding box is adjusted to the same aspect ratio as the aspect ratio of the div containing the board.
@@ -313,10 +313,17 @@ JXG.JSXGraph = {
             offX = 0, offY = 0,
             renderer, dimensions, bbox,
             attr, axattr, axattr_x, axattr_y,
+            options,
+            theme = {},
             board;
 
         attributes = attributes || {};
-        attr = this._setAttributes(attributes);
+        // Merge a possible theme
+        if (attributes.theme !== 'default' && Type.exists(JXG.themes[attributes.theme])) {
+            theme = JXG.themes[attributes.theme];
+        }
+        options = Type.deepCopy(Options, theme, true);
+        attr = this._setAttributes(attributes, options);
 
         dimensions = Env.getDimensions(box, attr.document);
 
@@ -343,8 +350,10 @@ JXG.JSXGraph = {
             // Size of HTML div.
             // If zero, the size is set to a small value to avoid
             // division by zero.
-            w = Math.max(parseInt(dimensions.width, 10), Mat.eps);
-            h = Math.max(parseInt(dimensions.height, 10), Mat.eps);
+            // w = Math.max(parseInt(dimensions.width, 10), Mat.eps);
+            // h = Math.max(parseInt(dimensions.height, 10), Mat.eps);
+            w = parseInt(dimensions.width, 10);
+            h = parseInt(dimensions.height, 10);
 
             if (Type.exists(bbox) && attr.keepaspectratio) {
                 /*
@@ -375,7 +384,8 @@ JXG.JSXGraph = {
         renderer = this.initRenderer(box, dimensions, attr.document, attr.renderer);
         this._setARIA(box, attr);
 
-        // create the board
+        // Create the board.
+        // board.options will contain the user supplied board attributes
         board = new Board(
             box,
             renderer,
@@ -394,14 +404,16 @@ JXG.JSXGraph = {
 
         this._fillBoard(board, attr, dimensions);
 
-        // create elements like axes, grid, navigation, ...
+        // Create elements like axes, grid, navigation, ...
         board.suspendUpdate();
+        attr = board.attr;
         if (attr.axis) {
             axattr = typeof attr.axis === "object" ? attr.axis : {};
 
             // The defaultAxes attributes are overwritten by user supplied axis object.
-            axattr_x = Type.deepCopy(Options.board.defaultAxes.x, axattr);
-            axattr_y = Type.deepCopy(Options.board.defaultAxes.y, axattr);
+            axattr_x = Type.deepCopy(options.board.defaultaxes.x, axattr);
+            axattr_y = Type.deepCopy(options.board.defaultaxes.y, axattr);
+
             // The user supplied defaultAxes attributes are merged in.
             if (attr.defaultaxes.x) {
                 axattr_x = Type.deepCopy(axattr_x, attr.defaultaxes.x);
@@ -411,22 +423,8 @@ JXG.JSXGraph = {
             }
 
             board.defaultAxes = {};
-            board.defaultAxes.x = board.create(
-                "axis",
-                [
-                    [0, 0],
-                    [1, 0]
-                ],
-                axattr_x
-            );
-            board.defaultAxes.y = board.create(
-                "axis",
-                [
-                    [0, 0],
-                    [0, 1]
-                ],
-                axattr_y
-            );
+            board.defaultAxes.x = board.create("axis", [[0, 0], [1, 0]], axattr_x);
+            board.defaultAxes.y = board.create("axis", [[0, 0], [0, 1]], axattr_y);
         }
         if (attr.grid) {
             board.create("grid", [], typeof attr.grid === "object" ? attr.grid : {});
@@ -439,7 +437,7 @@ JXG.JSXGraph = {
     /**
      * Load a board from a file containing a construction made with either GEONExT,
      * Intergeo, Geogebra, or Cinderella.
-     * @param {String} box HTML-ID to the HTML-element in which the board is painted.
+     * @param {String|Object} box id of or reference to the HTML element in which the board is painted.
      * @param {String} file base64 encoded string.
      * @param {String} format containing the file format: 'Geonext' or 'Intergeo'.
      * @param {Object} attributes Attributes for the board and 'encoding'.
@@ -506,7 +504,7 @@ JXG.JSXGraph = {
     /**
      * Load a board from a base64 encoded string containing a construction made with either GEONExT,
      * Intergeo, Geogebra, or Cinderella.
-     * @param {String} box HTML-ID to the HTML-element in which the board is painted.
+     * @param {String|Object} box id of or reference to the HTML element in which the board is painted.
      * @param {String} string base64 encoded string.
      * @param {String} format containing the file format: 'Geonext', 'Intergeo', 'Geogebra'.
      * @param {Object} attributes Attributes for the board and 'encoding'.
@@ -525,9 +523,9 @@ JXG.JSXGraph = {
         attributes = attributes || {};
         attr = this._setAttributes(attributes);
 
-            dimensions = Env.getDimensions(box, attr.document);
-            renderer = this.initRenderer(box, dimensions, attr.document, attr.renderer);
-            this._setARIA(box, attr);
+        dimensions = Env.getDimensions(box, attr.document);
+        renderer = this.initRenderer(box, dimensions, attr.document, attr.renderer);
+        this._setARIA(box, attr);
 
         /* User default parameters, in parse* the values in the gxt files are submitted to board */
         board = new Board(
@@ -551,7 +549,8 @@ JXG.JSXGraph = {
 
     /**
      * Delete a board and all its contents.
-     * @param {JXG.Board,String} board HTML-ID to the DOM-element in which the board is drawn.
+     * @param {JXG.Board|String} board id of or reference to the DOM element in which the board is drawn.
+     *
      */
     freeBoard: function (board) {
         var el;
