@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2023
+    Copyright 2008-2024
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -37,10 +37,10 @@
  * server side a python plugin system is used.
  */
 
-import JXG from "../jxg";
-import Zip from "../utils/zip";
-import Base64 from "../utils/base64";
-import Type from "../utils/type";
+import JXG from "../jxg.js";
+import Zip from "../utils/zip.js";
+import Base64 from "../utils/base64.js";
+import Type from "../utils/type.js";
 
 /**
  * @namespace
@@ -78,16 +78,17 @@ JXG.Server = {
      * @param {Boolean} sync If the call should be synchronous or not.
      */
     callServer: function (action, callback, data, sync) {
-        var fileurl, passdata, AJAX, params, id, dataJSONStr, k;
+        var fileurl, passdata, AJAX,// params, k,
+            id, dataJSONStr;
 
         sync = sync || false;
 
-        params = "";
-        for (k in data) {
-            if (data.hasOwnProperty(k)) {
-                params += "&" + escape(k) + "=" + escape(data[k]);
-            }
-        }
+        // params = "";
+        // for (k in data) {
+        //     if (data.hasOwnProperty(k)) {
+        //         params += "&" + escape(k) + "=" + escape(data[k]);
+        //     }
+        // }
 
         dataJSONStr = Type.toJSON(data);
 
