@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2023
+    Copyright 2008-2024
         Matthias Ehmann,
         Michael Gerhaeuser,
         Carsten Miller,
@@ -40,8 +40,9 @@
  * the type of a variable.
  */
 
-import JXG from "../jxg";
-import Const from "../base/constants";
+import JXG from "../jxg.js";
+import Const from "../base/constants.js";
+import Mat from "../math/math.js";
 
 JXG.extend(
     JXG,
@@ -104,17 +105,22 @@ JXG.extend(
         /**
          * Checks if the value of a given variable is of type number.
          * @param v A variable of any type.
-         * @param {Boolean} [dontAcceptNaN=false] If set to true, the function returns true for v=NaN.
+         * @param {Boolean} [acceptStringNumber=false] If set to true, the function returns true for e.g. v='3.1415'.
+         * @param {Boolean} [acceptNaN=true] If set to false, the function returns false for v=NaN.
          * @returns {Boolean} True, if v is of type number.
          */
-        isNumber: function (v, dontAcceptNaN) {
+        isNumber: function (v, acceptStringNumber, acceptNaN) {
             var result = (
-                typeof v === "number" || Object.prototype.toString.call(v) === "[Object Number]"
+                typeof v === 'number' || Object.prototype.toString.call(v) === '[Object Number]'
             );
-            dontAcceptNaN = dontAcceptNaN || false;
+            acceptStringNumber = acceptStringNumber || false;
+            acceptNaN = acceptNaN === undefined ? true : acceptNaN;
 
-            if(dontAcceptNaN) {
-                return result && !isNaN(v);
+            if (acceptStringNumber) {
+                result = result || ('' + parseFloat(v)) === v;
+            }
+            if (!acceptNaN) {
+                result = result && !isNaN(v);
             }
             return result;
         },
@@ -182,9 +188,14 @@ JXG.extend(
             return false;
         },
 
+        /**
+         * Checks if a given variable is a reference of a JSXGraph Point3D element.
+         * @param v A variable of any type.
+         * @returns {Boolean} True, if v is of type JXG.Point3D.
+         */
         isPoint3D: function (v) {
-            if (v !== null && typeof v === "object" && this.exists(v.elType)) {
-                return v.elType === "point3d";
+            if (v !== null && typeof v === "object" && this.exists(v.type)) {
+                return v.type === Const.OBJECT_TYPE_POINT3D;
             }
 
             return false;
@@ -213,6 +224,30 @@ JXG.extend(
             return this.isPoint(p);
         },
 
+        /**
+         * Checks if a given variable is a reference of a JSXGraph Point3D element or an array of length three
+         * or a function returning an array of length three.
+         * @param {JXG.Board} board
+         * @param v A variable of any type.
+         * @returns {Boolean} True, if v is of type JXG.Point3D or an array of length at least 3, or a function returning
+         * such an array.
+         */
+        isPointType3D: function (board, v) {
+            var val, p;
+
+            if (this.isArray(v) && v.length >= 3) {
+                return true;
+            }
+            if (this.isFunction(v)) {
+                val = v();
+                if (this.isArray(val) && val.length >= 3) {
+                    return true;
+                }
+            }
+            p = board.select(v);
+            return this.isPoint3D(p);
+        },
+
         /**
          * Checks if a given variable is a reference of a JSXGraph transformation element or an array
          * of JSXGraph transformation elements.
@@ -306,10 +341,50 @@ JXG.extend(
             return false;
         },
 
+        /**
+         * Converts a given CSS style string into a JavaScript object.
+         * @param {String} styles String containing CSS styles.
+         * @returns {Object} Object containing CSS styles.
+         */
+        cssParse: function (styles) {
+            var str = styles;
+            if (!this.isString(str)) return {};
+
+            str = str.replace(/\s*;\s*$/g, '');
+            str = str.replace(/\s*;\s*/g, '","');
+            str = str.replace(/\s*:\s*/g, '":"');
+            str = str.trim();
+            str = '{"' + str + '"}';
+
+            return JSON.parse(str);
+        },
+
+        /**
+         * Converts a given object into a CSS style string.
+         * @param {Object} styles Object containing CSS styles.
+         * @returns {String} String containing CSS styles.
+         */
+        cssStringify: function (styles) {
+            var str = '',
+                attr, val;
+            if (!this.isObject(styles)) return '';
+
+            for (attr in styles) {
+                if (!styles.hasOwnProperty(attr)) continue;
+                val = styles[attr];
+                if (!this.isString(val) && !this.isNumber(val)) continue;
+
+                str += attr + ':' + val + '; ';
+            }
+            str = str.trim();
+
+            return str;
+        },
+
         /**
          * Convert a String, a number or a function into a function. This method is used in Transformation.js
          * @param {JXG.Board} board Reference to a JSXGraph board. It is required to resolve dependencies given
-         * by a GEONE<sub>X</sub>T string, thus it must be a valid reference only in case one of the param
+         * by a JessieCode string, thus it must be a valid reference only in case one of the param
          * values is of type string.
          * @param {Array} param An array containing strings, numbers, or functions.
          * @param {Number} n Length of <tt>param</tt>.
@@ -321,7 +396,7 @@ JXG.extend(
                 deps = {};
 
             for (i = 0; i < n; i++) {
-                f[i] = JXG.createFunction(param[i], board, "", true);
+                f[i] = JXG.createFunction(param[i], board);
                 for (e in f[i].deps) {
                     deps[e] = f[i].deps;
                 }
@@ -339,37 +414,39 @@ JXG.extend(
          * Convert a String, number or function into a function.
          * @param {String|Number|Function} term A variable of type string, function or number.
          * @param {JXG.Board} board Reference to a JSXGraph board. It is required to resolve dependencies given
-         * by a GEONE<sub>X</sub>T string, thus it must be a valid reference only in case one of the param
+         * by a JessieCode/GEONE<sub>X</sub>T string, thus it must be a valid reference only in case one of the param
          * values is of type string.
          * @param {String} variableName Only required if function is supplied as JessieCode string or evalGeonext is set to true.
-         * Describes the variable name of the variable in a GEONE<sub>X</sub>T string given as term.
-         * @param {Boolean} [evalGeonext=true] Set this true, if term should be treated as a GEONE<sub>X</sub>T string.
-         * @returns {Function} A function evaluation the value given by term or null if term is not of type string,
+         * Describes the variable name of the variable in a JessieCode/GEONE<sub>X</sub>T string given as term.
+         * @param {Boolean} [evalGeonext=false] Obsolete and ignored! Set this true
+         * if term should be treated as a GEONE<sub>X</sub>T string.
+         * @returns {Function} A function evaluating the value given by term or null if term is not of type string,
          * function or number.
          */
         createFunction: function (term, board, variableName, evalGeonext) {
             var f = null;
 
-            if ((!this.exists(evalGeonext) || evalGeonext) && this.isString(term)) {
+            // if ((!this.exists(evalGeonext) || evalGeonext) && this.isString(term)) {
+            if (this.isString(term)) {
                 // Convert GEONExT syntax into  JavaScript syntax
                 //newTerm = JXG.GeonextParser.geonext2JS(term, board);
                 //return new Function(variableName,'return ' + newTerm + ';');
-
                 //term = JXG.GeonextParser.replaceNameById(term, board);
                 //term = JXG.GeonextParser.geonext2JS(term, board);
-                f = board.jc.snippet(term, true, variableName, true);
+
+                f = board.jc.snippet(term, true, variableName, false);
             } else if (this.isFunction(term)) {
                 f = term;
-                f.deps = {};
+                f.deps = (this.isObject(term.deps)) ? term.deps : {};
             } else if (this.isNumber(term)) {
                 /** @ignore */
                 f = function () { return term; };
                 f.deps = {};
-            } else if (this.isString(term)) {
-                // In case of string function like fontsize
-                /** @ignore */
-                f = function () { return term; };
-                f.deps = {};
+            // } else if (this.isString(term)) {
+            //     // In case of string function like fontsize
+            //     /** @ignore */
+            //     f = function () { return term; };
+            //     f.deps = {};
             }
 
             if (f !== null) {
@@ -427,7 +504,7 @@ JXG.extend(
                         attributes,
                         board.options,
                         attrClass,
-                        attrArray[j]
+                        attrArray[j].toLowerCase()
                     );
                 }
                 if (this.isArray(parents[i]) && parents[i].length > 1) {
@@ -877,6 +954,75 @@ JXG.extend(
             return str;
         },
 
+        /**
+         * Convert value v. If v has the form
+         * <ul>
+         * <li> 'x%': return floating point number x * percentOfWhat * 0.01
+         * <li> 'xfr': return floating point number x * percentOfWhat
+         * <li> 'xpx': return x * convertPx or convertPx(x) or x
+         * <li> x or 'x': return floating point number x
+         * </ul>
+         * @param {String|Number} v
+         * @param {Number} percentOfWhat
+         * @param {Function|Number|*} convertPx
+         * @returns {String|Number}
+         */
+        parseNumber: function(v, percentOfWhat, convertPx) {
+            var str;
+
+            if (this.isString(v) && v.indexOf('%') > -1) {
+                str = v.replace(/\s+%\s+/, '');
+                return parseFloat(str) * percentOfWhat * 0.01;
+            }
+            if (this.isString(v) && v.indexOf('fr') > -1) {
+                str = v.replace(/\s+fr\s+/, '');
+                return parseFloat(str) * percentOfWhat;
+            }
+            if (this.isString(v) && v.indexOf('px') > -1) {
+                str = v.replace(/\s+px\s+/, '');
+                str = parseFloat(str);
+                if(this.isFunction(convertPx)) {
+                    return convertPx(str);
+                } else if(this.isNumber(convertPx)) {
+                    return str * convertPx;
+                } else {
+                    return str;
+                }
+            }
+            // Number or String containing no unit
+            return parseFloat(v);
+        },
+
+        /**
+         * Parse a string for label positioning of the form 'left pos' or 'pos right'
+         * and return e.g.
+         * <tt>{ side: 'left', pos: 'pos' }</tt>.
+         * @param {String} str
+         * @returns {Obj}  <tt>{ side, pos }</tt>
+         */
+        parsePosition: function(str) {
+            var a, i,
+                side = '',
+                pos = '';
+
+            str = str.trim();
+            if (str !== '') {
+                a = str.split(/[ ,]+/);
+                for (i = 0; i < a.length; i++) {
+                    if (a[i] === 'left' || a[i] === 'right') {
+                        side = a[i];
+                    } else {
+                        pos = a[i];
+                    }
+                }
+            }
+
+            return {
+                side: side,
+                pos: pos
+            };
+        },
+
         /**
          * Extracts the keys of a given object.
          * @param object The object the keys are to be extracted
@@ -1028,6 +1174,13 @@ JXG.extend(
 
                         obj1[i] = this.merge(obj1[i], o);
                     } else {
+                        if (typeof obj1 === 'boolean') {
+                            // This is necessary in the following scenario:
+                            //   lastArrow == false
+                            // and call of
+                            //   setAttribute({lastArrow: {type: 7}})
+                            obj1 = {};
+                        }
                         obj1[i] = o;
                     }
                 }
@@ -1058,14 +1211,15 @@ JXG.extend(
                 c = [];
                 for (i = 0; i < obj.length; i++) {
                     prop = obj[i];
-                    if (typeof prop === "object") {
+                    // Attention: typeof null === 'object'
+                    if (prop !== null && typeof prop === "object") {
                         // We certainly do not want to recurse into a JSXGraph object.
                         // This would for sure result in an infinite recursion.
                         // As alternative we copy the id of the object.
                         if (this.exists(prop.board)) {
                             c[i] = prop.id;
                         } else {
-                            c[i] = this.deepCopy(prop);
+                            c[i] = this.deepCopy(prop, {}, toLower);
                         }
                     } else {
                         c[i] = prop;
@@ -1081,7 +1235,7 @@ JXG.extend(
                             if (this.exists(prop.board)) {
                                 c[i2] = prop.id;
                             } else {
-                                c[i2] = this.deepCopy(prop);
+                                c[i2] = this.deepCopy(prop, {}, toLower);
                             }
                         } else {
                             c[i2] = prop;
@@ -1094,9 +1248,9 @@ JXG.extend(
                         i2 = toLower ? i.toLowerCase() : i;
 
                         prop = obj2[i];
-                        if (typeof prop === "object") {
+                        if (prop !== null && typeof prop === "object") {
                             if (this.isArray(prop) || !this.exists(c[i2])) {
-                                c[i2] = this.deepCopy(prop);
+                                c[i2] = this.deepCopy(prop, {}, toLower);
                             } else {
                                 c[i2] = this.deepCopy(c[i2], prop, toLower);
                             }
@@ -1119,14 +1273,16 @@ JXG.extend(
          * @param {Object} attr Object with attributes - usually containing default options
          * @param {Object} special Special option values which overwrite (recursively) the default options
          * @param {Boolean} [toLower=true] If true the keys are converted to lower case.
+         * @param {Boolean} [ignoreUndefinedSpecials=false] If true the values in special that are undefined are not used.
          *
          * @see JXG#merge
          *
          */
-        mergeAttr: function (attr, special, toLower) {
+        mergeAttr: function (attr, special, toLower, ignoreUndefinedSpecials) {
             var e, e2, o;
 
             toLower = toLower || true;
+            ignoreUndefinedSpecials = ignoreUndefinedSpecials || false;
 
             for (e in special) {
                 if (special.hasOwnProperty(e)) {
@@ -1145,7 +1301,7 @@ JXG.extend(
                             attr[e2] = {};
                         }
                         this.mergeAttr(attr[e2], o, toLower);
-                    } else {
+                    } else if(!ignoreUndefinedSpecials || this.exists(o)) {
                         // Flat copy
                         // This is also used in the cases
                         //   attr.shadow = { enabled: true ...}
@@ -1158,6 +1314,47 @@ JXG.extend(
             }
         },
 
+        /**
+         * Convert a n object to a new object containing only
+         * lower case properties.
+         *
+         * @param {Object} obj
+         * @returns Object
+         * @example
+         * var attr = JXG.keysToLowerCase({radiusPoint: {visible: false}});
+         *
+         * // return {radiuspoint: {visible: false}}
+         */
+        keysToLowerCase: function (obj) {
+            var key, val,
+                keys = Object.keys(obj),
+                n = keys.length,
+                newObj = {};
+
+            if (typeof obj !== 'object') {
+                return obj;
+            }
+
+            while (n--) {
+                key = keys[n];
+                if (obj.hasOwnProperty(key)) {
+                    // We recurse into an object only if it is
+                    // neither a DOM node nor an JSXGraph object
+                    val = obj[key];
+                    if (typeof val === 'object' &&
+                        val !== null &&
+                        !this.isArray(val) &&
+                        !this.exists(val.nodeType) &&
+                        !this.exists(val.board)) {
+                        newObj[key.toLowerCase()] = this.keysToLowerCase(val);
+                    } else {
+                        newObj[key.toLowerCase()] = val;
+                    }
+                }
+            }
+            return newObj;
+        },
+
         /**
          * Generates an attributes object that is filled with default values from the Options object
          * and overwritten by the user specified attributes.
@@ -1167,7 +1364,7 @@ JXG.extend(
          * @returns {Object} The resulting attributes object
          */
         copyAttributes: function (attributes, options, s) {
-            var a, i, len, o, isAvail,
+            var a, arg, i, len, o, isAvail,
                 primitives = {
                     circle: 1,
                     curve: 1,
@@ -1195,12 +1392,15 @@ JXG.extend(
             }
 
             // Default options from the specific element like 'line' in
-            // copyAttribute(attributes, board.options, 'line')
+            //     copyAttribute(attributes, board.options, 'line')
+            // but also like in
+            //     Type.copyAttributes(attributes, board.options, 'view3d', 'az', 'slider');
             o = options;
             isAvail = true;
             for (i = 2; i < len; i++) {
-                if (this.exists(o[arguments[i]])) {
-                    o = o[arguments[i]];
+                arg = arguments[i];
+                if (this.exists(o[arg])) {
+                    o = o[arg];
                 } else {
                     isAvail = false;
                     break;
@@ -1212,14 +1412,16 @@ JXG.extend(
 
             // Merge the specific options given in the parameter 'attributes'
             // into the default options.
-            // Additionally, we step into a subelement of attribute like line.point1 in case it is supplied as in
-            // copyAttribute(attributes, board.options, 'line', 'point1')
+            // Additionally, we step into a sub-element of attribute like line.point1 -
+            // in case it is supplied as in
+            //     copyAttribute(attributes, board.options, 'line', 'point1')
             // In this case we would merge attributes.point1 into the global line.point1 attributes.
-            o = (typeof attributes === 'object') ? attributes : {};
+            o = (typeof attributes === 'object') ? this.keysToLowerCase(attributes) : {};
             isAvail = true;
             for (i = 3; i < len; i++) {
-                if (this.exists(o[arguments[i]])) {
-                    o = o[arguments[i]];
+                arg = arguments[i].toLowerCase();
+                if (this.exists(o[arg])) {
+                    o = o[arg];
                 } else {
                     isAvail = false;
                     break;
@@ -1238,17 +1440,18 @@ JXG.extend(
             o = options;
             isAvail = true;
             for (i = 2; i < len; i++) {
-                if (this.exists(o[arguments[i]])) {
-                    o = o[arguments[i]];
+                arg = arguments[i];
+                if (this.exists(o[arg])) {
+                    o = o[arg];
                 } else {
                     isAvail = false;
                     break;
                 }
             }
             if (isAvail && this.exists(o.label)) {
-                a.label = JXG.deepCopy(o.label, a.label);
+                a.label = JXG.deepCopy(o.label, a.label, true);
             }
-            a.label = JXG.deepCopy(options.label, a.label);
+            a.label = JXG.deepCopy(options.label, a.label, true);
 
             return a;
         },
@@ -1279,6 +1482,7 @@ JXG.extend(
          * @param {Object} obj A JavaScript object, functions will be ignored.
          * @param {Boolean} [noquote=false] No quotes around the name of a property.
          * @returns {String} The given object stored in a JSON string.
+         * @deprecated
          */
         toJSON: function (obj, noquote) {
             var list, prop, i, s, val;
@@ -1286,7 +1490,7 @@ JXG.extend(
             noquote = JXG.def(noquote, false);
 
             // check for native JSON support:
-            if (typeof JSON && JSON.stringify && !noquote) {
+            if (JSON !== undefined && JSON.stringify && !noquote) {
                 try {
                     s = JSON.stringify(obj);
                     return s;
@@ -1520,6 +1724,69 @@ JXG.extend(
             return str.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, "");
         },
 
+        /**
+         * Convert a floating point number to a string integer + fraction.
+         * Returns either a string of the form '3 1/3' (in case of useTeX=false)
+         * or '3 \\frac{1}{3}' (in case of useTeX=true).
+         *
+         * @param {Number} x
+         * @param {Boolean} [useTeX=false]
+         * @param {Number} [order=0.001]
+         * @returns {String}
+         * @see JXG.Math#decToFraction
+         */
+        toFraction: function (x, useTeX, order) {
+            var arr = Mat.decToFraction(x, order),
+                str = '';
+
+            if (arr[1] === 0 && arr[2] === 0) {
+                // 0
+                str += '0';
+            } else {
+                // Sign
+                if (arr[0] < 0) {
+                    str += '-';
+                }
+                if (arr[2] === 0) {
+                    // Integer
+                    str += arr[1];
+                } else if (!(arr[2] === 1 && arr[3] === 1)) {
+                    // Proper fraction
+                    if (arr[1] !== 0) {
+                        // Absolute value larger than 1
+                        str += arr[1] + ' ';
+                    }
+                    // Add fractional part
+                    if (useTeX === true) {
+                        str += '\\frac{' + arr[2] + '}{' + arr[3] + '}';
+                    } else {
+                        str += arr[2] + '/' + arr[3];
+                    }
+                }
+            }
+            return str;
+        },
+
+        /**
+         * Concat array src to array dest.
+         * Uses push instead of JavaScript concat, which is much
+         * faster.
+         * The array dest is changed in place.
+         * <p><b>Attention:</b> if "dest" is an anonymous array, the correct result is returned from the function.
+         *
+         * @param {Array} dest
+         * @param {Array} src
+         * @returns Array
+         */
+        concat: function(dest, src) {
+            var i,
+                le = src.length;
+            for (i = 0; i < le; i++) {
+                dest.push(src[i]);
+            }
+            return dest;
+        },
+
         /**
          * Convert HTML tags to entities or use html_sanitize if the google caja html sanitizer is available.
          * @param {String} str
@@ -1557,6 +1824,43 @@ JXG.extend(
             }
 
             return s;
+        },
+
+        /**
+         * Convert a string containing a MAXIMA /STACK expression into a JSXGraph / JessieCode string
+         * or an array of JSXGraph / JessieCode strings.
+         * <p>
+         * This function is meanwhile superseded by stack_jxg.stack2jsxgraph.
+         *
+         * @example
+         * console.log( JXG.stack2jsxgraph("%e**x") );
+         * // Output:
+         * //    "EULER**x"
+         *
+         * @example
+         * console.log( JXG.stack2jsxgraph("[%pi*(x**2 - 1), %phi*(x - 1), %gamma*(x+1)]") );
+         * // Output:
+         * //    [ "PI*(x**2 - 1)", "1.618033988749895*(x - 1)", "0.5772156649015329*(x+1)" ]
+         *
+         * @param {String} str
+         * @returns String
+         */
+        stack2jsxgraph: function(str) {
+            var t;
+
+            t = str.
+                replace(/%pi/g, 'PI').
+                replace(/%e/g, 'EULER').
+                replace(/%phi/g, '1.618033988749895').
+                replace(/%gamma/g, '0.5772156649015329').
+                trim();
+
+            // String containing array -> array containing strings
+            if (t[0] === '[' && t[t.length - 1] === ']') {
+                t = t.slice(1, -1).split(/\s*,\s*/);
+            }
+
+            return t;
         }
     }
 );
