@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2023
+    Copyright 2008-2024
         Matthias Ehmann,
         Carsten Miller,
         Andreas Walter,
@@ -36,6 +36,7 @@ describe("Test geometry functions", function () {
         axis: false,
         grid: false,
         boundingbox: [-8, 8, 8, -8],
+        resize: {enabled: false},
         showCopyright: false,
         showNavigation: false
     });
@@ -144,5 +145,83 @@ describe("Test geometry functions", function () {
         expect(i2.Y()).toBeCloseTo(2.046153846150052, 10);
     });
 
+    it("distPointSegment", function () {
+        var d;
+
+        d = JXG.Math.Geometry.distPointSegment([1, 2, 1], [1, -1, 0], [1,  1, 0]);
+        expect(d).toBeCloseTo(1.4142135623730951, 10);
+
+        d = JXG.Math.Geometry.distPointSegment([1, 1, 1], [1, 1, 0], [1,  1, 0]);
+        expect(d).toBeCloseTo(1.0, 10);
+
+        d = JXG.Math.Geometry.distPointSegment([1, 0, 1], [1, -1, 0], [1,  2, 0]);
+        expect(d).toBeCloseTo(1.0, 10);
+
+        d = JXG.Math.Geometry.distPointSegment([1, -1, 0], [1, -1, 0], [1,  2, 0]);
+        expect(d).toBeCloseTo(0.0, 10);
+
+        d = JXG.Math.Geometry.distPointSegment([1, -1, 1], [1, -1, 0], [1,  2, 0]);
+        expect(d).toBeCloseTo(1.0, 10);
+
+    });
+
+    it("Circle radius by slider", function () {
+        var d, slider, circle;
+
+        slider = board.create('slider', [[4, -3], [4, 3], [-4, -1, 4]], { name: 'a'});
+        circle = board.create('circle', [[-1, 0], 1], {});
+        circle.setRadius('a');
+
+        d = circle.Radius();
+        expect(d).toEqual(1.0);
+    });
+
+    it("Circle radius by slider, nonnegative", function () {
+        var d, slider, circle;
+
+        slider = board.create('slider', [[4, -3], [4, 3], [-4, -1, 4]], { name: 'a'});
+        circle = board.create('circle', [[-1, 0], 1], {
+            nonnegativeOnly: true
+        });
+        circle.setRadius('a');
+
+        d = circle.Radius();
+        expect(d).toEqual(0.0);
+    });
+
+    it("Segment radius by slider", function () {
+        var d, slider, seg;
+
+        slider = board.create('slider', [[4, -3], [4, 3], [-4, -1, 4]]);
+        var seg = board.create('segment', [[-4, 3], [0, 3], () => slider.Value()], {
+            point1: {visible: true},
+            point2: {visible: true}
+        });
+
+        d = seg.L();
+        expect(d).toEqual(1.0);
+    });
+
+    it("Segment radius by slider, nonnegative", function () {
+        var d, slider, seg;
+
+        slider = board.create('slider', [[4, -3], [4, 3], [-4, -1, 4]]);
+        var seg = board.create('segment', [[-4, 3], [0, 3], () => slider.Value()], {
+            point1: {visible: true},
+            point2: {visible: true},
+            nonnegativeOnly: true
+        });
+
+        d = seg.L();
+        expect(d).toEqual(0.0);
+    });
+
+    it("Glider on arc", function () {
+        var sector, glider;
+        sector = board.create('arc', [[-1, -1], [3, 0], [-4, 0]]);
+        glider = board.create('glider', [-2, 3, sector]);
+
+        expect(glider.position).toBeCloseTo(0.6100503447261109, 10);
+    });
 
 });
